unit OOUse;

{$mode objfpc}{$H+}

interface

uses
   SysUtils, Classes, Graphics, Controls,
   Forms, Dialogs, StdCtrls, ComObj, Variants;

   type
   TOpenOffice = class
      function Connect: boolean;
      procedure Disconnect;
      function CreateDocument: boolean;
      function OpenDocument(const aFileUrl:string): boolean;
      procedure SaveDocument(const aFileUrl:string);
      procedure CloseDocument(const Flag:boolean);
      procedure InsertText(const myText:String);
   private
      StarOffice: Variant;
      StarDesktop: variant;
      Document: Variant;
      function MakePropertyValue(PropName:string; PropValue:variant):variant;
   end;

implementation

function TOpenOffice.MakePropertyValue(PropName:string; PropValue:variant):variant;
var struct : variant;
begin
    Struct := StarOffice.Bridge_GetStruct('com.sun.star.beans.PropertyValue');
    variant(Struct.Name) := PropName;
    variant(Struct.Value) := PropValue;
    Result := Struct;
end;


function TOpenOffice.Connect: boolean;
begin
   if VarIsEmpty(StarOffice) then
      StarOffice := CreateOleObject('com.sun.star.ServiceManager');
   StarDesktop := StarOffice.createInstance('com.sun.star.frame.Desktop');
   Result := not (VarIsEmpty(StarOffice) or VarIsNull(StarOffice));
end;

procedure TOpenOffice.Disconnect;
begin
   StarOffice := Unassigned;
end;

function TOpenOffice.CreateDocument: boolean;
begin
   Document := StarDesktop.LoadComponentFromURL('private:factory/swriter', '_blank', 0, VarArrayCreate([0, -1], varVariant));
   Result := not (VarIsEmpty(Document) or VarIsNull(Document));
end;

function TOpenOffice.OpenDocument(const aFileUrl:string): boolean;
begin
   Document := StarDesktop.LoadComponentFromURL(aFileUrl, '_blank', 0, VarArrayCreate([0, - 1], varVariant));
   Result := not (VarIsEmpty(Document) or VarIsNull(Document));
end;

procedure TOpenOffice.SaveDocument(const aFileUrl:string);
var
   VariantArr: variant;
begin
   VariantArr := VarArrayCreate([0, 0], varVariant);
   VariantArr[0] := MakePropertyValue('FilterName', 'MS Word 97');
   Document.StoreToURL(aFileUrl, VariantArr);
end;


procedure TOpenOffice.InsertText(const myText:String);
var
   oCursor: Variant;
   oText: Variant;
begin
//get document text object
  oText := Document.GetText;
//create cursor
  oCursor := oText.CreateTextCursor;
//set some text properties
  oCursor.SetPropertyValue('CharColor', 255);
  oCursor.SetPropertyValue('CharShadowed', True);
//insert string
  oText.InsertString(oCursor, myText, false);
//insert line break character
  oText.InsertControlCharacter(oCursor, 0, false);

end;

procedure TOpenOffice.CloseDocument(const Flag:boolean);
begin
  Document.Close(Flag);
  StarDesktop.Terminate;
end;

end.


